#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

import pyssdb


class SsdbQueue(object):
    """Simple Queue with SSDB Backend"""

    def __init__(self, name, **ssdb_kwargs):
        """The default connection parameters are:
        host='localhost', port=8888"""
        self.__db = pyssdb.Client(**ssdb_kwargs)
        self.key = name

    def qsize(self):
        """Return the approximate size of the queue."""
        return self.__db.qsize(self.key)

    def empty(self):
        """Return True if the queue is empty, False otherwise."""
        return self.qsize() == 0

    def put(self, item):
        """Put item into the queue."""
        self.__db.qpush(self.key, item)

    def get(self):
        """Remove and return an item from the queue.

        If optional args block is true and timeout is None (the default), block
        if necessary until an item is available."""

        item = self.__db.qpop(self.key)

        return item

    def clean(self):
        """Empty key"""
        return self.__db.qclear(self.key)

    def db(self):
        return self.__db



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Author: wuyue
# Email: wuyue92tree@163.com


from __future__ import print_function

import email
import traceback
import imaplib
from imapclient import IMAPClient
from email.header import decode_header
from retrying import retry


imaplib._MAXLINE = 10000000

SEEN = br'\Seen'


class MailReceiver(object):
    def __init__(self, host=None, username=None, password=None,
                 port=None, SSL=True):
        self.server = IMAPClient(host=host, ssl=SSL, timeout=100)
        self.username = username
        self.password = password

    @retry(stop_max_attempt_number=3)
    def folder_list(self):
        folders = self.server.list_folders()

        res_list = []
        for folder in folders:
            if folder:
                # print(folder[2].encode("utf-8"))
                res_list.append(folder[2].encode("utf-8"))
        return res_list

    @retry(stop_max_attempt_number=3)
    def message_list(self, mailbox='INBOX', search_='all'):
        self.server.select_folder(
            mailbox)
        # message_list = self.server.search('ON 21-Dec-2017')
        message_list = self.server.search(search_)
        return message_list

    @retry(stop_max_attempt_number=3)
    def download_message_list(self, message_list):
        msg_data = self.server.fetch(
            message_list, ['INTERNALDATE', 'FLAGS', 'BODY.PEEK[]'])
        if not msg_data:
            # print(self.username)
            return None, None
        return message_list, msg_data

    def get_content(self, message):
        try:
            while True:
                res = {}
                msg = email.message_from_string(message['BODY[]'])
                for s, c in decode_header(msg['subject']):
                    try:
                        res['subject'] = s.decode(c, 'ignore')
                    except TypeError:
                        res['subject'] = s
                        # print('Subject: ' + res['subject'])
                        # print('From: ' + email.utils.parseaddr(msg['from'])[1])
                        # print('To: ' + email.utils.parseaddr(msg['to'])[1])

                for par in msg.walk():
                    if not par.is_multipart():
                        name = par.get_param("name")
                        if name:
                            # print(name)
                            pass
                        else:
                            body = par.get_payload(decode=True)
                            if not body:
                                continue
                            try:
                                code = par.get_content_charset()
                                res['body'] = body.decode(code, 'ignore')
                            except TypeError:
                                res['body'] = body

                # print('____________________________________________')
                return res

        except Exception as e:
            traceback.print_exc()
            # print('got msg error: %s' % e)
            return

    def delete_message(self, messages, deleted_folder=u"Deleted Messages"):
        try:
            self.server.add_flags(messages, SEEN)
            if deleted_folder:
                # 将邮件移动到 已删除
                self.server.copy(messages, deleted_folder)
            self.server.delete_messages(messages)
            self.server.expunge()
            return True
        except Exception as e:
            # print("邮件删除失败： %s" % e)
            traceback.print_exc()
            return

    def close(self):
        self.server.logout()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: chaojiying.py
@create at: 2018-05-11 16:33

这一行开始写关于本文件的说明与解释
"""

import requests
from hashlib import md5


class ChaoJiYingApi(object):
    def __init__(self, username, password, soft_id):
        self.username = username
        self.password = md5(password).hexdigest()
        self.soft_id = soft_id
        self.base_params = {
            'user': self.username,
            'pass2': self.password,
            'softid': self.soft_id,
        }
        self.headers = {
            'Connection': 'Keep-Alive',
            'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0;'
                          ' Windows NT 5.1; Trident/4.0)',
        }

    def post_pic(self, im, code_type):
        """
        im: 图片字节
        codetype: 题目类型 参考 http://www.chaojiying.com/price.html
        """
        params = {
            'codetype': code_type,
        }
        params.update(self.base_params)
        files = {'userfile': ('ccc.jpg', im)}
        r = requests.post('http://upload.chaojiying.net/Upload/Processing.php',
                          data=params, files=files, headers=self.headers)
        return r.json()

    def report_error(self, im_id):
        """
        im_id:报错题目的图片ID
        """
        params = {
            'id': im_id,
        }
        params.update(self.base_params)
        r = requests.post(
            'http://upload.chaojiying.net/Upload/ReportError.php', data=params,
            headers=self.headers)
        return r.json()

    def decode(self, img_path, code_type):
        im = open(img_path, 'rb').read()
        res = self.post_pic(im, code_type)
        # {u'err_str': u'OK', u'err_no': 0,
        #  u'md5': u'a11171f1f444e8d1992926f4ca16c7d8',
        #  u'pic_id': u'6031116291508600001',
        #  u'pic_str': u'113,72|220,81|138,101'}
        if res.get('err_no') == 0 and res.get('err_str') == u'OK':
            return res.get('pic_str')
        return



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: dupefilters.py
@create at: 2018-06-14 14:52

这一行开始写关于本文件的说明与解释
"""

import logging
import time
import datetime
import hashlib
from crwy.utils.filter.RedisSet import RedisSet
from crwy.utils.filter.RedisSortedSet import RedisSortedSet
from scrapy.dupefilters import BaseDupeFilter
from scrapy.exceptions import NotConfigured
from scrapy_redis.connection import get_redis_from_settings

logger = logging.getLogger(__name__)


class RedisRFPDupeFilter(BaseDupeFilter):
    """
    dupefilter by redis, redis connect base on scrapy-redis connect

    warning:
        config SPIDER_NAME in settings before use
    default:
        DUPEFILTER_DEBUG = False
        DUPEFILTER_DELAY_DAY = 0
    """
    logger = logger

    def __init__(self, debug=False,
                 server=None,
                 bot_name=None,
                 spider_name=None,
                 duperliter_delay_day=None,
                 do_hash=None):
        self.debug = debug
        self.logdupes = True
        self.server = server
        self.bot_name = bot_name
        self.spider_name = spider_name
        self.duperliter_delay_day = duperliter_delay_day
        self.do_hash = do_hash

    @classmethod
    def from_settings(cls, settings):
        server = get_redis_from_settings(settings)
        debug = settings.getbool('DUPEFILTER_DEBUG')
        bot_name = settings.get('BOT_NAME')
        spider_name = settings.get('SPIDER_NAME')
        duperliter_delay_day = settings.getint('DUPEFILTER_DELAY_DAY', 0)
        do_hash = settings.getbool('DUPEFILTER_DO_HASH', True)
        if not spider_name:
            raise NotConfigured('%s - "SPIDER_NAME" is not found.' %
                                cls.__name__)
        return cls(debug=debug, server=server, bot_name=bot_name,
                   spider_name=spider_name,
                   duperliter_delay_day=duperliter_delay_day,
                   do_hash=do_hash)

    def request_seen(self, request):
        if not request.meta.get('dupefilter_key', None):
            return False

        if len(request.meta.get('redirect_urls', [])) > 0:
            # skip url from redirect
            return False

        dupefilter_key = request.meta.get('dupefilter_key')
        dupefilter_key = hashlib.sha1(dupefilter_key).hexdigest() if \
            self.do_hash else dupefilter_key

        # SPIDER_NAME for dupefilter
        key = '{bot_name}:{spider_name}'.format(
            bot_name=self.bot_name,
            spider_name=self.spider_name)

        if request.meta.get('duperliter_delay_day', ''):
            self.duperliter_delay_day = int(request.meta.get(
                'duperliter_delay_day'))

        if self.duperliter_delay_day == 0:
            s = RedisSet(key, server=self.server)
            if s.sadd(dupefilter_key) is True:
                return False
            self.logger.info('Filtered dupefilter_key: %s' %
                             dupefilter_key)
            return True
        else:
            z = RedisSortedSet(key, server=self.server)
            now = time.time()
            last_time = z.zscore(dupefilter_key)

            if not last_time:
                z.zadd(now, dupefilter_key)
                return False

            if (datetime.datetime.utcfromtimestamp(now) -
                datetime.datetime.utcfromtimestamp(last_time)).days >= \
                    self.duperliter_delay_day:
                z.zadd(now, dupefilter_key)
                return False
            self.logger.info('Filtered dupefilter_key within %s day(s): %s' %
                             (self.duperliter_delay_day,
                              request.meta.get('dupefilter_key')))
            return True

    def log(self, request, spider):  # log that a request has been filtered
        if self.debug:
            msg = "Filtered duplicate request: %(request)s"
            self.logger.debug(msg, {
                'request': request.meta.get('dupefilter_key')}, extra={
                'spider': spider})
        elif self.logdupes:
            msg = ("Filtered duplicate request: %(request)s"
                   " - no more duplicates will be shown"
                   " (see DUPEFILTER_DEBUG to show all duplicates)")
            self.logger.debug(msg, {'request': request},
                              extra={'spider': spider})
            self.logdupes = False

        spider.crawler.stats.inc_value('dupefilter/filtered', spider=spider)


class ReleaseDupefilterKey(object):
    """
    rm dupefilter_key from redis, when call response
    """

    def call(self, spider, dupefilter_key):

        if not dupefilter_key:
            return

        obj = RedisRFPDupeFilter().from_settings(spider.settings)

        dupefilter_key = hashlib.sha1(dupefilter_key).hexdigest() if \
            obj.do_hash else dupefilter_key

        # SPIDER_NAME for dupefilter
        key = '{bot_name}:{spider_name}'.format(
            bot_name=obj.bot_name,
            spider_name=obj.spider_name)

        if obj.duperliter_delay_day == 0:
            s = RedisSet(key, server=obj.server)
            s.srem(dupefilter_key)
        else:
            z = RedisSortedSet(key, server=obj.server)
            z.zrem(dupefilter_key)
        obj.logger.info('dupefilter_key: {} released.'.format(
            dupefilter_key))


release_dupefilter_key = ReleaseDupefilterKey()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: yima.py
@create at: 2017-10-27 09:57

这一行开始写关于本文件的说明与解释
"""

from __future__ import print_function, unicode_literals

from crwy.spider import Spider
from crwy.exceptions import CrwyException


class YiMa(Spider):
    def __init__(self, username, password, item_id):
        super(YiMa, self).__init__()
        if username and password and item_id:
            self.username = username
            self.password = password
            self.item_id = item_id
        else:
            raise CrwyException("[YiMa] params not valid.")

    def login(self):
        """
        YiMa 登录
        :return: 登录token
        """

        try:
            url = "http://api.fxhyd.cn/UserInterface.aspx?" \
                  "action=login&username={username}" \
                  "&password={password}".format(username=self.username,
                                                password=self.password)
            res = self.html_downloader.download(url)

            if 'success' not in res.text:
                raise CrwyException("[YiMa] Login failed.")

            return res.text.strip().split("|")[-1]
        except Exception as e:
            raise CrwyException(e)

    def get_phone(self, token, phone_type='',
                  phone='', not_prefix=''):
        """
        获取手机号
        :param token:   登录token
        :param phone_type:  运营商 1 [移动] 2 [联通] 3 [电信]
        :param phone:   指定号码
        :param not_prefix:  不要号段 (例子:notPrefix=170.177 ,代表不获取170和177的号段)
        :return: 手机号码
        """
        try:
            url = "http://api.fxhyd.cn/UserInterface.aspx?action=getmobile&" \
                  "token={token}&itemid={item_id}&excludeno=" \
                  "{not_prefix}&isp={phone_type}&" \
                  "mobile={phone}".format(token=token, item_id=self.item_id,
                                          not_prefix=not_prefix,
                                          phone_type=phone_type, phone=phone)

            res = self.html_downloader.download(url)
            if 'success' not in res.text:
                raise CrwyException("[YiMa] get phone failed.")

            # print(res.text)
            return res.text.strip().split('|')[-1]

        except Exception as e:
            raise CrwyException(e)

    def get_message(self, token, phone):
        """
        获取短信消息
        :param token:   登录token
        :param phone:   手机号
        :return:
        """
        try:
            url = "http://api.fxhyd.cn/UserInterface.aspx?action=getsms&" \
                  "token={token}&itemid={item_id}&mobile={phone}" \
                  "&release=0".format(token=token, item_id=self.item_id,
                                      phone=phone)
            res = self.html_downloader.download(url)

            if 'success' not in res.text:
                raise CrwyException("[YiMa] get message failed.")

            else:
                return res.text.strip().split('|')[-1]

        except Exception as e:
            raise CrwyException(e)

    def release_phone(self, token, phone):
        try:
            url = "http://api.fxhyd.cn/UserInterface.aspx?action=release&" \
                  "token={token}&itemid={item_id}&mobile={phone}" \
                  "&release=0".format(token=token, item_id=self.item_id,
                                      phone=phone)
            res = self.html_downloader.download(url)

            if 'success' not in res.text:
                raise CrwyException("[YiMa] release phone failed.")

        except Exception as e:
            raise CrwyException(e)

    def add_black(self, token, phone):
        try:
            url = "http://api.fxhyd.cn/UserInterface.aspx?action=addignore&" \
                  "token={token}&itemid={item_id}&mobile={phone}" \
                  "&release=0".format(token=token, item_id=self.item_id,
                                      phone=phone)
            res = self.html_downloader.download(url)

            if 'success' not in res.text:
                raise CrwyException("[YiMa] black phone failed.")

        except Exception as e:
            raise CrwyException(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from __future__ import print_function
import sys
import gevent
import threading
from optparse import OptionParser
from crwy.commands.list import Command as ListCommand


class Command(object):
    def execute(self, spider_name):
        module = __import__('src.%s' % spider_name)
        cls_obj = getattr(
            getattr(module, spider_name), spider_name.capitalize() + 'Spider')

        spider = cls_obj()
        res = spider.run()
        return res

    def multi_coroutine(self, spider_name, coroutine):
        try:
            coroutine = int(coroutine)
        except ValueError:
            print('ERROR: coroutine must be int!!!')
            sys.exit(1)

        from gevent import monkey
        monkey.patch_all()

        gevent.joinall([
            gevent.spawn(self.execute, spider_name) for i in
            xrange(coroutine)
        ])

    def multi_thread(self, spider_name, thread):
        try:
            thread = int(thread)
        except ValueError:
            print('ERROR: thread must be int!!!')
            sys.exit(1)

        thread_list = []
        for i in xrange(thread):
            t = threading.Thread(target=self.execute, args=(spider_name,))
            t.start()
            thread_list.append(t)

        for t in thread_list:
            t.join()

    def main(self):
        Usage = "Usage:  crwy runspider [option] [args]"
        parser = OptionParser(Usage)
        parser.add_option(
            '-n', '--name', dest='name', help='spider name', metavar="NAME")
        parser.add_option(
            '-c', '--coroutine', dest='coroutine',
            help='crawler by multi coroutine', metavar="COROUTINE")
        parser.add_option(
            '-t', '--thread', dest='thread',
            help='crawler by multi thread', metavar="THREAD")
        opt, args = parser.parse_args()

        if opt.name is not None:
            if opt.name in ListCommand.get_spider_list():
                sys.path.append('.')

                if opt.coroutine and opt.thread:
                    print("Can not run use both coroutine and thread!")
                    sys.exit(1)

                if opt.coroutine is not None:
                    self.multi_coroutine(opt.name, opt.coroutine)
                elif opt.thread is not None:
                    self.multi_thread(opt.name, opt.thread)
                else:
                    self.execute(opt.name)
            else:
                print('ERROR spider: "%s" is not found!!!' % opt.name)
                sys.exit(1)
        else:
            print(Usage)
            sys.exit(1)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: settings.py
@create at: 2018-06-20 19:33

这一行开始写关于本文件的说明与解释
"""

from crwy.utils.load_settings import LoadSettingsFromConsul
from crwy.exceptions import CrwyException


class ScrapySettingsFromConsul(LoadSettingsFromConsul):
    def __init__(self, spider_name, bot_name, prefix='scrapy', **kwargs):
        super(ScrapySettingsFromConsul, self).__init__(**kwargs)
        self.spider_name = spider_name
        self.bot_name = bot_name
        self.prefix = prefix

    def init_main_key(self, key=None):
        if not key:
            self.main_key = '{prefix}/{bot_name}/{spider_name}'.format(
                prefix=self.prefix, bot_name=self.bot_name,
                spider_name=self.spider_name
            )
        else:
            self.main_key = key

    def _get_settings(self, key=None):
        self.init_main_key(key=key)
        index, data = self.c.kv.get(self.main_key, recurse=True)
        if not data:
            raise CrwyException('Please make sure the key: <%s> is exist.' %
                                self.main_key)

        new_data = {
            item.get('Key').split('/')[-1]: eval(item.get('Value'))
            for item in data
        }
        new_data['SPIDER_NAME'] = self.spider_name

        return new_data

    @classmethod
    def get_settings(cls, spider_name, bot_name, key=None, prefix='scrapy',
                     **kwargs):
        load_settings = cls(spider_name, bot_name, prefix=prefix, **kwargs)
        return load_settings._get_settings(key=key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from crwy.utils.no_sql.redis_m import get_redis_client


class RedisSortedSet(object):
    """Simple Sorted Deduplicate with Redis Backend"""

    def __init__(self, name, namespace='deduplicate_sorted', server=None,
                 **redis_kwargs):
        """
        The default connection parameters are:
        host='localhost', port=6379, db=0
        """
        if server:
            self.__db = server
        else:
            self.__db = get_redis_client(**redis_kwargs)
        self.key = '%s:%s' % (namespace, name)

    def zadd(self, score, item):
        """Add item."""
        if self.__db.zadd(self.key, score, item) == 0:
            return False
        else:
            return True

    def zrem(self, item):
        """Del item."""
        if self.__db.zrem(self.key, item) == 0:
            return False
        else:
            return True

    def zcard(self):
        """Return total count."""
        return self.__db.zcard(self.key)

    def zscore(self, item):
        """Return item score."""
        return self.__db.zscore(self.key, item)

    def zmembers(self):
        """Return all item."""
        return self.__db.zmembers(self.key)

    def clean(self):
        """Empty key"""
        return self.__db.delete(self.key)

    def db(self):
        return self.__db



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from __future__ import print_function
import sys
import os

from crwy.commands.runspider import Command as RunspiderCommand
from crwy.commands.startproject import Command as StartprojectCommand
from crwy.commands.createspider import Command as CreatespiderCommand
from crwy.commands.list import Command as ListCommand
from crwy.settings.default_settings import BASE_DIR

try:
    import ConfigParser
except ImportError:
    from configparser import ConfigParser


def under_conf():
    if os.path.exists('crwy.cfg'):
        return True
    else:
        return False


def get_project_name():
    conf = ConfigParser()
    conf.read('crwy.cfg', encoding='utf-8')
    project_name = conf.get('project', 'project_name').encode('utf-8')
    return project_name


def get_project_settings():
    conf = ConfigParser()
    conf.read('crwy.cfg', encoding='utf-8')
    settings = conf.get('settings', 'default').encode('utf-8')
    return settings


def execute():
    if under_conf():
        Header = 'Crwy - project: %s \n' % get_project_name()
    else:
        Header = 'Crwy - no active project found!!!\n'

    Usage = 'Usage:\n  crwy <commands> [option] [args]\n'
    project_list = ['runspider', 'startproject', 'createspider', 'list',
                    'version']
    Options = 'Avaliable Commands:\n' \
              '  list\t\tlist all spider in your project\n' \
              '  runspider\trun a spider\n' \
              '  startproject\tcreate a new project\n' \
              '  createspider\tcreate a new spider\n' \
              '  version\tshow version\n'

    Notice = 'Use "crwy <command> -h" to see more info about a command'
    # print sys.argv
    if len(sys.argv) < 2:
        print(Header)
        print(Usage)
        print(Options)
        print(Notice)
        sys.exit(1)

    project = sys.argv[1]

    if project not in project_list:
        print(Header)
        print(Usage)
        print(Options)
        print(Notice)
        sys.exit(1)

    if project == 'startproject':
        cmd = StartprojectCommand()
        cmd.main()

    if project == 'runspider':
        if under_conf():
            cmd = RunspiderCommand()
            cmd.main()
        else:
            print('[ERROR] Please makesure that you are under project path!!!')

    if project == 'createspider':
        if under_conf():
            cmd = CreatespiderCommand()
            cmd.main()
        else:
            print('[ERROR] Please makesure that you are under project path!!!')

    if project == 'list':
        if under_conf():
            cmd = ListCommand()
            cmd.main()
        else:
            print('[ERROR] Please makesure that you are under project path!!!')

    if project == 'version':
        version_path = os.path.join(BASE_DIR, 'VERSION')
        print(open(version_path, 'r').read())


if __name__ == '__main__':
    execute()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: load_settings.py
@create at: 2018-06-20 19:32

这一行开始写关于本文件的说明与解释
"""

import consul
from crwy.exceptions import CrwyException


class LoadSettingsFromConsul(object):
    def __init__(self, **kwargs):
        self.c = consul.Consul(**kwargs)
        self.main_key = None

    def init_main_key(self, key=None):
        if not key:
            raise CrwyException('Please set key first.')
        self.main_key = key

    def _get_settings(self, key=None):
        self.init_main_key(key=key)
        index, data = self.c.kv.get(self.main_key, recurse=True)
        if not data:
            raise CrwyException('Please make sure the key: <%s> is exist.' %
                                self.main_key)

        new_data = {
            item.get('Key').split('/')[-1]: eval(item.get('Value'))
            for item in data
        }

        return new_data

    @classmethod
    def get_settings(cls, key=None, **kwargs):
        load_settings = cls(**kwargs)
        return load_settings._get_settings(key=key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from __future__ import print_function

import sys
import os
from optparse import OptionParser
from crwy.settings.default_settings import TEMPLATE_DIR
from crwy.changetmpl import change_spider_name
from crwy.commands.list import Command as ListCommand

PATH = os.path.join(TEMPLATE_DIR, 'spider')
TMPLATE_PATH = os.path.join(PATH, '%s.py.tmpl')
SPIDER_TMPL_LIST = ['basic', 'sqlite', 'queue', 'redis_queue']


class Command(object):
    def create_spider(self, name, tmpl):
        spider_list = ListCommand.get_spider_list()
        if name in spider_list:
            print('[ERROR] spider "%s" is exists!!!' % name)
            sys.exit(1)

        path = TMPLATE_PATH % tmpl
        spider = change_spider_name(name, path)
        f = open('src/%s.py' % name, 'w')
        f.write(spider)
        print('Spider is base on : %s template' % tmpl)
        print('Create spider : "%s" success!!!' % name)

    def main(self):
        Usage = "Usage:  crwy createspider [option] [args]"
        parser = OptionParser(Usage)
        parser.add_option(
            '-l',
            '--list',
            action='store_true',
            dest='list',
            help='list available spider template name',
            metavar="LIST")
        parser.add_option(
            '-p',
            '--preview',
            dest='preview',
            help='preview spider template',
            metavar="PREVIEW")
        parser.add_option(
            '-t',
            '--tmpl',
            dest='template',
            help='spider template',
            metavar="TEMPLATE")
        parser.add_option(
            '-n',
            '--name',
            dest='name',
            help='new spider name',
            metavar="NAME")

        opt, args = parser.parse_args()
        # print(opt)
        if not opt.name and not opt.list and not opt.template and not opt.preview:
            print(Usage)
            sys.exit(1)

        if opt.list:
            print('Available template:')
            for template in SPIDER_TMPL_LIST:
                print('  %s' % template)
            sys.exit(1)

        if opt.preview is not None:
            if opt.preview in SPIDER_TMPL_LIST:
                print(open(TMPLATE_PATH % opt.preview, 'r').read())
                sys.exit(1)
            else:
                print('[ERROR] Illegal template name!!!')
                sys.exit(1)

        if opt.template is not None:
            if opt.template in SPIDER_TMPL_LIST:
                tmpl = opt.template
            else:
                print('[ERROR] Illegal template name!!!')
                sys.exit(1)
        else:
            tmpl = 'basic'

        if opt.name is not None:
            name = opt.name
        else:
            print('[ERROR] Please enter a spider name!!!')
            sys.exit(1)

        self.create_spider(name, tmpl)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: common.py
@create at: 2017-12-12 18:01

这一行开始写关于本文件的说明与解释
"""

import os
import re
import datetime

try:
    import ConfigParser
except ImportError:
    from configparser import ConfigParser

__all__ = [
    'cookie2str', 'cookie2dict', 'config_handle',
    'file_handle', 'datetime2str', 'str2datetime',
    'dict2obj', 'obj2dict', 'remove_emoji', 'change_kv'
]


def cookie2str(cookie_dict):
    """
    将requests 字典类型cookie转换成字符串
    :param cookie_dict: dict
    :return: string
    """
    return '; '.join(
        [name + '=' + cookie_dict.get(name) for name in cookie_dict])


def cookie2dict(cookie_str):
    """
    将cookie_str转换成requests可用的dict类型
    :param cookie_str: string
    :return: dict
    """
    cookie_dict = dict()
    for item in cookie_str.strip().replace(' ', '').split(';'):
        if not item:
            continue
        name, value = item.split('=', 1)
        cookie_dict[name] = value
    return cookie_dict


def datetime2str(target, fmt='%Y-%m-%d %H:%M:%S'):
    """
    将datetime对象转换成字符串
    :param target: datetime
    :param fmt: string
    :return: string
    """
    return datetime.datetime.strftime(target, fmt)


def str2datetime(target, fmt='%Y-%m-%d %H:%M:%S'):
    """
    将string转换成datetime对象
    :param target: string
    :param fmt: string
    :return: datetime
    """
    return datetime.datetime.strptime(target, fmt)


def dict2obj(target, change_dict=True):
    """
    将dict转换成obj对象
    change_dict 用于控制是否转换target内部dict为obj

    :param target: dict
    :param change_dict: bool
    :return: obj
    """

    class Obj(object):
        def __init__(self, d, change_dict):
            for a, b in d.items():
                if change_dict is True:
                    if isinstance(b, (list, tuple)):
                        setattr(self, a,
                                [Obj(x, change_dict) if isinstance(x, dict) else x
                                 for x in b])
                    else:
                        setattr(self, a, Obj(b, change_dict) if isinstance(
                            b, dict) else b)
                else:
                    setattr(self, a, b)

    return Obj(target, change_dict=change_dict)


def obj2dict(target):
    """
    将obj对象转换成dict
    :param target: obj
    :return: dict
    """
    return target.__dict__


def config_handle(path):
    """
    用于对Config配置文件进行操作，初始化config_path
    :param path: config文件路径
    :return: 返回config对象
    """
    config = ConfigParser.ConfigParser()
    config.read(path)
    return config


def file_handle(path, file_name, mode='r'):
    """
    用于对普通文件进行操作
    :param path: 文件路径
    :param file_name: 文件名称
    :param mode: 加载模式，默认'r'
    :return: file对象
    """
    if path[-1] == '/':
        real_path = path + file_name
    else:
        real_path = path + '/' + file_name

    if not os.path.exists(path):
        os.makedirs(path)

    return open(real_path, mode=mode)


def remove_emoji(content):
    """
    表情符去除
    :param content: unicode
    :return: unicode
    """
    pattern = re.compile(
        u"(\ud83d[\ude00-\ude4f])|"  # emoticons
        u"(\ud83c[\udf00-\uffff])|"  # symbols & pictographs (1 of 2)
        u"(\ud83d[\u0000-\uddff])|"  # symbols & pictographs (2 of 2)
        u"(\ud83d[\ude80-\udeff])|"  # transport & map symbols
        u"(\ud83c[\udde0-\uddff])"  # flags (iOS)
        "+", flags=re.UNICODE)
    return pattern.sub(r'', content)


def change_kv(dict_ori):
    """
    字典kv调换
    :param dict_ori: 原字典
    :return: 新字典
    """
    return dict(zip(dict_ori.values(), dict_ori.keys()))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os.path import dirname, join
from setuptools import setup, find_packages

with open(join(dirname(__file__), 'crwy/VERSION'), 'rb') as f:
    version = f.read().decode('ascii').strip()

setup(
    name='Crwy',
    version=version,
    url='https://github.com/wuyue92tree/crwy',
    description='A Simple Web Crawling and Web Scraping framework',
    long_description=open('README.md').read(),
    author='wuyue',
    author_email='wuyue92tree@163.com',
    maintainer='wuyue',
    maintainer_email='wuyue92tree@163.com',
    license='MIT',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': ['crwy = crwy.cmdline:execute']
    },
    install_requires=[
        'beautifulsoup4>=4.5.1',
        'requests>=2.20.0',
        'configparser>=3.5.0',
        'gevent>=1.2.1'
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

import inspect
from crwy.utils.html.html_downloader import HtmlDownloader
from crwy.utils.html.html_parser import HtmlParser
from crwy.utils.logger import Logger


class BaseSpider(object):
    """ Spider基础类 """
    def __init__(self):
        """
        初始化下载器/解析器及日志接口
        """
        self.html_downloader = HtmlDownloader()
        self.html_parser = HtmlParser()


class Spider(BaseSpider):
    """ Spider类 提供基本方法 """
    def __init__(self, logger=None):
        super(Spider, self).__init__()
        self.login_kwargs = None    # 用于存放登录时所需的参数
        self.proxies = None
        if logger:
            self.logger = logger
        else:
            self.logger = Logger.timed_rt_logger()

    def login(self, *args, **kwargs):
        pass

    def clean(self, *args, **kwargs):
        pass

    def save(self, *args, **kwargs):
        pass

    def get_cookie(self):
        pass

    @staticmethod
    def func_name():
        """ 返回函数名称 """
        return inspect.stack()[1][3]
